@@ -1,4 +1,4 @@
-# Copyrights 2009 by Mark Overmeer.
+# Copyrights 2009-2010 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 1.06.
@@ -7,15 +7,17 @@ use strict;
 
 package XML::Compile::SOAP12;
 use vars '$VERSION';
-$VERSION = '2.00';
+$VERSION = '2.01';
 
 use base 'XML::Compile::SOAP';
 
 use Log::Report 'xml-compile-soap12', syntax => 'SHORT';
 
-use XML::Compile::Util;
+use XML::Compile::Util          qw/SCHEMA2001/;
 use XML::Compile::SOAP::Util;
+
 use XML::Compile::SOAP12::Util;
+use XML::Compile::SOAP12::Operation;
 
 my %roles =
  ( NEXT     => SOAP12NEXT
@@ -24,13 +26,6 @@ my %roles =
  );
 my %rev_roles = reverse %roles;
 
-XML::Compile->addSchemaDirs(__FILE__);
-XML::Compile->knownNamespace
- ( &SOAP12ENC => '2003-soap-encoding.xsd'
- , &SOAP12ENV => '2003-soap-envelope.xsd'
- , &SOAP12RPC => '2003-soap-rpc.xsd'
- );
-
 
 sub new($@)
 {   my $class = shift;
@@ -39,13 +34,30 @@ sub new($@)
 
 sub init($)
 {   my ($self, $args) = @_;
-    $args->{version}     = 'SOAP12';
     $self->SUPER::init($args);
+    $self->_initSOAP12($self->schemas);
+}
+
+sub _initSOAP12($)
+{   my ($self, $schemas) = @_;
+    return $self
+        if exists $schemas->prefixes->{env};
+
+    $schemas->importDefinitions([SOAP12ENC, SOAP12ENV, SOAP12RPC]);
+    $schemas->addKeyRewrite('PREFIXES(soap12)');
+
+    $schemas->prefixes
+      ( soap12env => SOAP12ENV  # preferred names by spec
+      , soap12enc => SOAP12ENC
+      , xsd       => SCHEMA2001
+      );
 
-    $self->schemas->importDefinitions( [SOAP12ENC, SOAP12ENV, SOAP12RPC] );
     $self;
 }
 
+sub version    { 'SOAP12' }
+sub envelopeNS { SOAP12ENV }
+
 
 sub sender($)
 {   my ($self, $args) = @_;
